"use strict";

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _enzyme = require("enzyme");

var _enzyme2 = _interopRequireDefault(_enzyme);

var _enzymeAdapterReact = require("enzyme-adapter-react-16.3");

var _enzymeAdapterReact2 = _interopRequireDefault(_enzymeAdapterReact);

var _Form = require("./components/Form");

var _Form2 = _interopRequireDefault(_Form);

var _Button = require("./components/Button");

var _Button2 = _interopRequireDefault(_Button);

var _chai = require("chai");

var _chai2 = _interopRequireDefault(_chai);

var _chaiEnzyme = require("chai-enzyme");

var _chaiEnzyme2 = _interopRequireDefault(_chaiEnzyme);

var _FormFragment = require("./components/FormFragment");

var _FormFragment2 = _interopRequireDefault(_FormFragment);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_chai2.default.use((0, _chaiEnzyme2.default)());
_enzyme2.default.configure({ adapter: new _enzymeAdapterReact2.default() });

// We want to create a custom renderer that takes numbers as input values but displays them as strings...
var customRenderer = function customRenderer(field, _onChange, onFieldFocus) {
  var _field$disabled = field.disabled,
      disabled = _field$disabled === undefined ? false : _field$disabled,
      errorMessages = field.errorMessages,
      id = field.id,
      isValid = field.isValid,
      name = field.name,
      placeholder = field.placeholder,
      required = field.required,
      type = field.type,
      value = field.value,
      label = field.label;

  var items = void 0;
  switch (type) {
    case "date":
      {
        // $FlowFixMe - this is OK, we know the value type...
        var convertedValue = value ? new Date(value).toUTCString() : "";
        return _react2.default.createElement(
          "div",
          { key: id },
          _react2.default.createElement(
            "label",
            { htmlFor: id },
            label,
            " "
          ),
          _react2.default.createElement("input", {
            type: "text",
            id: id,
            name: name,
            value: convertedValue,
            placeholder: placeholder,
            disabled: disabled,
            required: required,
            onChange: function onChange(evt) {
              return _onChange(id, new Date(evt.target.value).getTime());
            },
            onFocus: function onFocus() {
              return onFieldFocus(id);
            }
          }),
          !isValid ? _react2.default.createElement(
            "span",
            { className: "errors" },
            errorMessages
          ) : _react2.default.createElement("span", null)
        );
      }
  }
};

// Set up some values for comparing dates. We want to be able to pass in number values to the renderer but have
// them displayed as strings and for string values to be output to the form as numbers...
var xmasString = "Tue, 25 Dec 2018 00:00:00 GMT";
var xmasDate = new Date(xmasString);
var xmasNumber = xmasDate.getTime();

var newYearsEveString = "Mon, 31 Dec 2018 00:00:00 GMT";
var newYearsEveDate = new Date(newYearsEveString);
var newYearsEveNumber = newYearsEveDate.getTime();

var newYearString = "Tue, 01 Jan 2019 00:00:00 GMT";
var newYearDate = new Date(newYearString);
var newYearNumber = newYearDate.getTime();

describe("custom renderer", function () {
  var fields = [{
    id: "DATE1",
    name: "date1",
    type: "date",
    defaultValue: xmasNumber
  }];
  var form = (0, _enzyme.mount)(_react2.default.createElement(_Form2.default, { defaultFields: fields, renderer: customRenderer }));
  var field = form.find("input");

  test("field parses number to date", function () {
    expect(field.prop("value")).toBe(xmasString);
  });

  test("field outputs string value as number", function () {
    expect(form.state().value.date1).toBe(xmasNumber);
  });

  test("updated field value is coverted to a string", function () {
    field.prop("onChange")({ target: { value: newYearString } });
    expect(form.state().value.date1).toBe(newYearNumber);
  });
});

var firstDateError = "Must be before second date";
var secondDateError = "Must be after first date";

describe("compare date fields", function () {
  var fields = [{
    id: "DATE1",
    name: "date1",
    type: "date",
    validWhen: {
      comparedTo: {
        fields: ["DATE2"],
        is: "SMALLER",
        message: firstDateError
      }
    }
  }, {
    id: "DATE2",
    name: "date2",
    type: "date",
    validWhen: {
      comparedTo: {
        fields: ["DATE1"],
        is: "BIGGER",
        message: secondDateError
      }
    }
  }];
  var form = (0, _enzyme.mount)(_react2.default.createElement(_Form2.default, { defaultFields: fields, renderer: customRenderer }));
  var firstDate = form.find("input").at(0);
  var secondDate = form.find("input").at(1);

  test("form is initially invalid because no dates are set", function () {
    expect(form.state().isValid).toBe(false);

    firstDate.prop("onFocus")();
    secondDate.prop("onFocus")();
    form.update();
    expect(form.state().fields[0].errorMessages).toBe(firstDateError);
    expect(form.state().fields[1].errorMessages).toBe(secondDateError);
  });

  test("warning shown for fields", function () {
    expect(form.find("span.errors").length).toBe(2);
  });

  test("first date becomes valid when set, but form is still invalid because second date is not set", function () {
    firstDate.prop("onChange")({ target: { value: xmasString } });
    expect(form.state().isValid).toBe(false);
  });

  test("second date becomes valid when set and form is valid because both dates are set", function () {
    secondDate.prop("onChange")({ target: { value: newYearsEveString } });
    expect(form.state().isValid).toBe(true);
  });

  test("fields have no errors in form state", function () {
    expect(form.state().fields[0].isValid).toBe(true);
    expect(form.state().fields[0].errorMessages).toBe("");

    expect(form.state().fields[1].isValid).toBe(true);
    expect(form.state().fields[1].errorMessages).toBe("");
  });

  test("setting first date to date after second date causes fields to be invalid", function () {
    firstDate.prop("onChange")({ target: { value: newYearString } });
    expect(form.state().isValid).toBe(false);
    expect(form.state().fields[0].isValid).toBe(false);
    expect(form.state().fields[0].errorMessages).toBe(firstDateError);
  });

  test("updating both dates so first date is before second date makes the form valid", function () {
    firstDate.prop("onChange")({ target: { value: newYearsEveString } });
    secondDate.prop("onChange")({ target: { value: newYearString } });
    expect(form.state().isValid).toBe(true);
    expect(form.state().fields[0].errorMessages).toBe("");
  });
});